@@ -1,63 +1,75 @@
-from typing import Dict
+from typing import (Dict,
+                    Iterable)
 
-from robust.angular import (Orientation,
-                            orientation)
-from robust.linear import segments_intersection
-
-from orient.hints import (Multisegment,
+from ground.base import (Context,
+                         Orientation,
+                         Relation)
+from ground.hints import (Multisegment,
                           Point,
                           Segment)
-from . import bounding_box
+
+from . import box
+from .events_queue import LinearEventsQueue
+from .hints import SegmentEndpoints
 from .processing import process_open_linear_queue
-from .relation import Relation
-from .segment import (relate_point as relate_point_to_segment,
-                      relate_segment as relate_segments)
-from .sweep import OpenSweeper
-from .utils import flatten
+from .segment import (_relate_segment as relate_segments,
+                      relate_point as relate_point_to_segment)
+from .utils import to_sorted_pair
 
 
-def relate_point(multisegment: Multisegment, point: Point) -> Relation:
+def relate_point(multisegment: Multisegment, point: Point,
+                 *,
+                 context: Context) -> Relation:
     return (Relation.DISJOINT
-            if all(relate_point_to_segment(segment, point) is Relation.DISJOINT
-                   for segment in multisegment)
+            if all(relate_point_to_segment(segment, point,
+                                           context=context)
+                   is Relation.DISJOINT
+                   for segment in multisegment.segments)
             else Relation.COMPONENT)
 
 
-def relate_segment(multisegment: Multisegment, segment: Segment) -> Relation:
+def relate_segment(multisegment: Multisegment, segment: Segment,
+                   *,
+                   context: Context) -> Relation:
     all_components = has_no_touch = has_no_cross = has_no_overlap = True
     # orientations of multisegment's segments
     # which touch given segment in the middle
     middle_touching_orientations = {}  # type: Dict[Point, Orientation]
     components = []
-    start, end = segment
+    start, end = segment_endpoints = segment.start, segment.end
     if start > end:
         start, end = end, start
-    for index, sub_segment in enumerate(multisegment):
-        relation = relate_segments(sub_segment, segment)
+    for index, sub_segment in enumerate(multisegment.segments):
+        sub_segment_start, sub_segment_end = sub_segment_endpoints = (
+            sub_segment.start, sub_segment.end)
+        relation = relate_segments(sub_segment_start, sub_segment_end, start,
+                                   end,
+                                   context=context)
         if relation is Relation.COMPONENT:
             return Relation.COMPONENT
         elif relation is Relation.EQUAL:
             return (Relation.EQUAL
-                    if all_components and index == len(multisegment) - 1
+                    if (all_components
+                        and index == len(multisegment.segments) - 1)
                     else Relation.COMPONENT)
         elif relation is Relation.COMPOSITE:
             if has_no_overlap:
                 has_no_overlap = False
-            if start in sub_segment:
-                start = max(sub_segment)
-                segment = start, end
-            elif end in sub_segment:
-                end = min(sub_segment)
-                segment = start, end
+            if start in sub_segment_endpoints:
+                start = max(sub_segment_endpoints)
+                segment_endpoints = start, end
+            elif end in sub_segment_endpoints:
+                end = min(sub_segment_endpoints)
+                segment_endpoints = start, end
             else:
-                components.append(sort_endpoints(sub_segment))
+                components.append(to_sorted_pair(sub_segment_endpoints))
         elif relation is Relation.OVERLAP:
             if all_components:
                 all_components = False
             if has_no_overlap:
                 has_no_overlap = False
-            start, end = segment = _subtract_segments_overlap(segment,
-                                                              sub_segment)
+            start, end = segment_endpoints = _subtract_segments_overlap(
+                    segment_endpoints, sub_segment_endpoints)
         else:
             if all_components:
                 all_components = False
@@ -66,10 +78,10 @@ def relate_segment(multisegment: Multisegment, segment: Segment) -> Relation:
                     if has_no_touch:
                         has_no_touch = False
                     if has_no_cross:
-                        intersection = segments_intersection(sub_segment,
-                                                             segment)
+                        intersection = context.segments_intersection(
+                                sub_segment_start, sub_segment_end, start, end)
                         if intersection != start and intersection != end:
-                            sub_start, sub_end = sub_segment
+                            sub_start, sub_end = sub_segment_endpoints
                             non_touched_endpoint = (sub_start
                                                     if intersection == sub_end
                                                     else sub_end)
@@ -78,11 +90,11 @@ def relate_segment(multisegment: Multisegment, segment: Segment) -> Relation:
                                     middle_touching_orientations[intersection])
                             except KeyError:
                                 middle_touching_orientations[intersection] = (
-                                    orientation(end, start,
-                                                non_touched_endpoint))
+                                    context.angle_orientation(
+                                            start, end, non_touched_endpoint))
                             else:
-                                if (orientation(end, start,
-                                                non_touched_endpoint)
+                                if (context.angle_orientation(
+                                        start, end, non_touched_endpoint)
                                         is not previous_orientation):
                                     has_no_cross = False
                 elif has_no_cross and relation is Relation.CROSS:
@@ -120,34 +132,37 @@ def relate_segment(multisegment: Multisegment, segment: Segment) -> Relation:
                 else Relation.OVERLAP)
 
 
-def _subtract_segments_overlap(minuend: Segment,
-                               subtrahend: Segment) -> Segment:
+def _subtract_segments_overlap(minuend: SegmentEndpoints,
+                               subtrahend: SegmentEndpoints
+                               ) -> SegmentEndpoints:
     left_start, left_end, right_start, right_end = sorted(minuend + subtrahend)
     return ((left_start, left_end)
             if left_start in minuend
             else (right_start, right_end))
 
 
-def sort_endpoints(segment: Segment) -> Segment:
-    start, end = segment
-    return (segment
-            if start < end
-            else (end, start))
-
-
-def relate_multisegment(goal: Multisegment, test: Multisegment) -> Relation:
-    if not (goal and test):
+def relate_multisegment(goal: Multisegment, test: Multisegment,
+                        *,
+                        context: Context) -> Relation:
+    if not (goal.segments and test.segments):
         return Relation.DISJOINT
     goal_bounding_box, test_bounding_box = (
-        bounding_box.from_points(flatten(goal)),
-        bounding_box.from_points(flatten(test)))
-    if bounding_box.disjoint_with(goal_bounding_box, test_bounding_box):
+        box.from_multisegment(goal,
+                              context=context),
+        box.from_multisegment(test,
+                              context=context))
+    if box.disjoint_with(goal_bounding_box, test_bounding_box):
         return Relation.DISJOINT
-    sweeper = OpenSweeper()
-    sweeper.register_segments(goal,
-                              from_test=False)
-    sweeper.register_segments(test,
-                              from_test=True)
-    (_, goal_max_x, _, _), (_, test_max_x, _, _) = (goal_bounding_box,
-                                                    test_bounding_box)
-    return process_open_linear_queue(sweeper, min(goal_max_x, test_max_x))
+    events_queue = LinearEventsQueue(context)
+    events_queue.register(to_segments_endpoints(goal),
+                          from_test=False)
+    events_queue.register(to_segments_endpoints(test),
+                          from_test=True)
+    return process_open_linear_queue(events_queue,
+                                     min(goal_bounding_box.max_x,
+                                         test_bounding_box.max_x))
+
+
+def to_segments_endpoints(multisegment: Multisegment
+                          ) -> Iterable[SegmentEndpoints]:
+    return ((segment.start, segment.end) for segment in multisegment.segments)
