@@ -1,35 +1,42 @@
 from typing import Iterable
 
-from orient.hints import (Contour,
-                          Multiregion,
+from ground.base import (Context,
+                         Relation)
+from ground.hints import (Contour,
                           Multisegment,
                           Point,
                           Polygon,
-                          Region,
                           Segment)
-from . import bounding
+
+from . import box
+from .events_queue import CompoundEventsQueue
+from .hints import (Multiregion,
+                    Region,
+                    SegmentEndpoints)
 from .multiregion import (_relate_contour as relate_contour_to_multiregion,
                           _relate_region as relate_region_to_regions,
                           relate_multiregion as relate_multiregions,
                           relate_segment as relate_segment_to_multiregion,
-                          to_oriented_segments
+                          to_oriented_edges_endpoints
                           as multiregion_to_oriented_segments)
+from .multisegment import to_segments_endpoints
 from .processing import process_linear_compound_queue
 from .region import (_relate_contour as relate_contour_to_region,
                      _relate_region as relate_regions,
                      relate_point as relate_point_to_region,
                      relate_segment as relate_segment_to_region,
                      to_oriented_segments as region_to_oriented_segments)
-from .relation import Relation
-from .sweep import CompoundSweeper
 
 
-def relate_point(polygon: Polygon, point: Point) -> Relation:
-    border, holes = polygon
-    relation_without_holes = relate_point_to_region(border, point)
+def relate_point(polygon: Polygon, point: Point,
+                 *,
+                 context: Context) -> Relation:
+    relation_without_holes = relate_point_to_region(polygon.border, point,
+                                                    context=context)
     if relation_without_holes is Relation.WITHIN:
-        for hole in holes:
-            relation_with_hole = relate_point_to_region(hole, point)
+        for hole in polygon.holes:
+            relation_with_hole = relate_point_to_region(hole, point,
+                                                        context=context)
             if relation_with_hole is Relation.WITHIN:
                 return Relation.DISJOINT
             elif relation_with_hole is Relation.COMPONENT:
@@ -37,12 +44,16 @@ def relate_point(polygon: Polygon, point: Point) -> Relation:
     return relation_without_holes
 
 
-def relate_segment(polygon: Polygon, segment: Segment) -> Relation:
-    border, holes = polygon
-    relation_without_holes = relate_segment_to_region(border, segment)
+def relate_segment(polygon: Polygon, segment: Segment,
+                   *,
+                   context: Context) -> Relation:
+    border, holes = polygon.border, polygon.holes
+    relation_without_holes = relate_segment_to_region(border, segment,
+                                                      context=context)
     if (holes and (relation_without_holes is Relation.WITHIN
                    or relation_without_holes is Relation.ENCLOSED)):
-        relation_with_holes = relate_segment_to_multiregion(holes, segment)
+        relation_with_holes = relate_segment_to_multiregion(holes, segment,
+                                                            context=context)
         if relation_with_holes is Relation.DISJOINT:
             return relation_without_holes
         elif relation_with_holes is Relation.TOUCH:
@@ -58,35 +69,43 @@ def relate_segment(polygon: Polygon, segment: Segment) -> Relation:
 
 
 def relate_multisegment(polygon: Polygon,
-                        multisegment: Multisegment) -> Relation:
-    if not multisegment:
+                        multisegment: Multisegment,
+                        *,
+                        context: Context) -> Relation:
+    if not multisegment.segments:
         return Relation.DISJOINT
-    border, _ = polygon
-    polygon_bounding_box = bounding.box_from_iterable(border)
-    multisegment_bounding_box = bounding.box_from_iterables(multisegment)
-    if bounding.box_disjoint_with(polygon_bounding_box,
-                                  multisegment_bounding_box):
+    polygon_bounding_box = box.from_polygon(polygon,
+                                            context=context)
+    multisegment_bounding_box = box.from_multisegment(multisegment,
+                                                      context=context)
+    if box.disjoint_with(polygon_bounding_box,
+                         multisegment_bounding_box):
         return Relation.DISJOINT
-    sweeper = CompoundSweeper()
-    sweeper.register_segments(to_oriented_segments(polygon),
-                              from_test=False)
-    sweeper.register_segments(multisegment,
-                              from_test=True)
-    _, multisegment_max_x, _, _ = multisegment_bounding_box
-    _, polygon_max_x, _, _ = polygon_bounding_box
-    return process_linear_compound_queue(sweeper, min(multisegment_max_x,
-                                                      polygon_max_x))
+    events_queue = CompoundEventsQueue(context)
+    events_queue.register(to_oriented_edges_endpoints(polygon,
+                                                      context=context),
+                          from_test=False)
+    events_queue.register(to_segments_endpoints(multisegment),
+                          from_test=True)
+    return process_linear_compound_queue(events_queue,
+                                         min(multisegment_bounding_box.max_x,
+                                             polygon_bounding_box.max_x))
 
 
-def relate_contour(polygon: Polygon, contour: Contour) -> Relation:
-    border, holes = polygon
-    contour_bounding_box = bounding.box_from_iterable(contour)
-    relation_without_holes = relate_contour_to_region(border, contour,
-                                                      contour_bounding_box)
+def relate_contour(polygon: Polygon, contour: Contour,
+                   *,
+                   context: Context) -> Relation:
+    contour_bounding_box = box.from_contour(contour,
+                                            context=context)
+    relation_without_holes = relate_contour_to_region(polygon.border, contour,
+                                                      contour_bounding_box,
+                                                      context=context)
+    holes = polygon.holes
     if holes and (relation_without_holes is Relation.ENCLOSED
                   or relation_without_holes is Relation.WITHIN):
         relation_with_holes = relate_contour_to_multiregion(
-                holes, contour, contour_bounding_box)
+                holes, contour, contour_bounding_box,
+                context=context)
         if relation_with_holes is Relation.DISJOINT:
             return relation_without_holes
         elif relation_with_holes is Relation.TOUCH:
@@ -103,12 +122,17 @@ def relate_contour(polygon: Polygon, contour: Contour) -> Relation:
         return relation_without_holes
 
 
-def relate_region(polygon: Polygon, region: Region) -> Relation:
-    border, holes = polygon
-    region_bounding_box = bounding.box_from_iterable(region)
+def relate_region(polygon: Polygon, region: Region,
+                  *,
+                  context: Context) -> Relation:
+    region_bounding_box = box.from_contour(region,
+                                           context=context)
+    border, holes = polygon.border, polygon.holes
     relation_with_border = relate_regions(border, region,
-                                          bounding.box_from_iterable(border),
-                                          region_bounding_box)
+                                          box.from_contour(border,
+                                                           context=context),
+                                          region_bounding_box,
+                                          context=context)
     if relation_with_border in (Relation.DISJOINT,
                                 Relation.TOUCH,
                                 Relation.OVERLAP,
@@ -122,7 +146,8 @@ def relate_region(polygon: Polygon, region: Region) -> Relation:
                 else relation_with_border)
     else:
         relation_with_holes = relate_region_to_regions(holes, region,
-                                                       region_bounding_box)
+                                                       region_bounding_box,
+                                                       context=context)
         if relation_with_holes is Relation.DISJOINT:
             return relation_with_border
         elif relation_with_holes is Relation.TOUCH:
@@ -137,24 +162,33 @@ def relate_region(polygon: Polygon, region: Region) -> Relation:
             return Relation.OVERLAP
 
 
-def relate_multiregion(polygon: Polygon, multiregion: Multiregion) -> Relation:
-    return (_relate_multiregion(polygon, multiregion)
+def relate_multiregion(polygon: Polygon, multiregion: Multiregion,
+                       *,
+                       context: Context) -> Relation:
+    return (_relate_multiregion(polygon, multiregion,
+                                context=context)
             if multiregion
             else Relation.DISJOINT)
 
 
 def _relate_multiregion(polygon: Polygon,
-                        multiregion: Multiregion) -> Relation:
-    border, holes = polygon
-    border_bounding_box = bounding.box_from_iterable(border)
+                        multiregion: Multiregion,
+                        *,
+                        context: Context) -> Relation:
+    border, holes = polygon.border, polygon.holes
+    border_bounding_box = box.from_contour(border,
+                                           context=context)
     if not holes:
         return relate_region_to_regions(multiregion, border,
-                                        border_bounding_box).complement
+                                        border_bounding_box,
+                                        context=context).complement
     none_touch = True
     subsets_regions_indices = []
     for region_index, region in enumerate(multiregion):
         region_relation = relate_regions(border, region, border_bounding_box,
-                                         bounding.box_from_iterable(region))
+                                         box.from_contour(region,
+                                                          context=context),
+                                         context=context)
         if region_relation is Relation.TOUCH:
             if none_touch:
                 none_touch = False
@@ -176,7 +210,8 @@ def _relate_multiregion(polygon: Polygon,
                 holes,
                 multiregion
                 if is_subset_of_border
-                else [multiregion[index] for index in subsets_regions_indices])
+                else [multiregion[index] for index in subsets_regions_indices],
+                context=context)
         if relation_with_holes is Relation.DISJOINT:
             return ((Relation.WITHIN
                      if none_touch
@@ -205,31 +240,26 @@ def _relate_multiregion(polygon: Polygon,
                 else Relation.TOUCH)
 
 
-def relate_polygon(goal: Polygon, test: Polygon) -> Relation:
-    goal_border, goal_holes = goal
-    test_border, test_holes = test
-    return _relate_polygon(goal_border, goal_holes, test_border, test_holes,
-                           bounding.box_from_iterable(goal_border),
-                           bounding.box_from_iterable(test_border))
-
-
-def _relate_polygon(goal_border: Region,
-                    goal_holes: Multiregion,
-                    test_border: Region,
-                    test_holes: Multiregion,
-                    goal_border_bounding_box: bounding.Box,
-                    test_border_bounding_box: bounding.Box
-                    ) -> Relation:
+def relate_polygon(goal: Polygon, test: Polygon,
+                   *,
+                   context: Context) -> Relation:
+    goal_bounding_box, test_bounding_box = (box.from_polygon(goal,
+                                                             context=context),
+                                            box.from_polygon(test,
+                                                             context=context))
+    goal_border, goal_holes = goal.border, goal.holes
+    test_border, test_holes = test.border, test.holes
     borders_relation = relate_regions(goal_border, test_border,
-                                      goal_border_bounding_box,
-                                      test_border_bounding_box)
+                                      goal_bounding_box, test_bounding_box,
+                                      context=context)
     if borders_relation in (Relation.DISJOINT,
                             Relation.TOUCH,
                             Relation.OVERLAP):
         return borders_relation
     elif borders_relation is Relation.EQUAL:
         if goal_holes and test_holes:
-            holes_relation = relate_multiregions(test_holes, goal_holes)
+            holes_relation = relate_multiregions(test_holes, goal_holes,
+                                                 context=context)
             if holes_relation in (Relation.DISJOINT,
                                   Relation.TOUCH,
                                   Relation.OVERLAP):
@@ -256,8 +286,10 @@ def _relate_polygon(goal_border: Region,
             subsets_holes_indices = []
             for hole_index, hole in enumerate(goal_holes):
                 hole_relation = relate_regions(
-                        test_border, hole, test_border_bounding_box,
-                        bounding.box_from_iterable(hole))
+                        test_border, hole, test_bounding_box,
+                        box.from_contour(hole,
+                                         context=context),
+                        context=context)
                 if hole_relation is Relation.TOUCH:
                     if none_touch:
                         none_touch = False
@@ -277,7 +309,8 @@ def _relate_polygon(goal_border: Region,
                         goal_holes
                         if len(subsets_holes_indices) == len(goal_holes)
                         else [goal_holes[index]
-                              for index in subsets_holes_indices])
+                              for index in subsets_holes_indices],
+                        context=context)
                 if holes_relation is Relation.EQUAL:
                     return (Relation.ENCLOSED
                             if borders_relation is Relation.WITHIN
@@ -301,9 +334,11 @@ def _relate_polygon(goal_border: Region,
         none_touch = True
         subsets_holes_indices = []
         for hole_index, hole in enumerate(test_holes):
-            hole_relation = relate_regions(
-                    goal_border, hole, goal_border_bounding_box,
-                    bounding.box_from_iterable(hole))
+            hole_relation = relate_regions(goal_border, hole,
+                                           goal_bounding_box,
+                                           box.from_contour(hole,
+                                                            context=context),
+                                           context=context)
             if hole_relation is Relation.TOUCH:
                 if none_touch:
                     none_touch = False
@@ -323,7 +358,8 @@ def _relate_polygon(goal_border: Region,
                     test_holes
                     if len(subsets_holes_indices) == len(test_holes)
                     else [test_holes[index]
-                          for index in subsets_holes_indices])
+                          for index in subsets_holes_indices],
+                    context=context)
             if holes_relation is Relation.EQUAL:
                 return (Relation.ENCLOSES
                         if borders_relation is Relation.COVER
@@ -345,11 +381,14 @@ def _relate_polygon(goal_border: Region,
                 else borders_relation)
 
 
-def to_oriented_segments(polygon: Polygon,
-                         *,
-                         clockwise: bool = False) -> Iterable[Segment]:
-    border, holes = polygon
-    yield from region_to_oriented_segments(border,
-                                           clockwise=clockwise)
-    yield from multiregion_to_oriented_segments(holes,
-                                                clockwise=not clockwise)
+def to_oriented_edges_endpoints(polygon: Polygon,
+                                *,
+                                clockwise: bool = False,
+                                context: Context
+                                ) -> Iterable[SegmentEndpoints]:
+    yield from region_to_oriented_segments(polygon.border,
+                                           clockwise=clockwise,
+                                           context=context)
+    yield from multiregion_to_oriented_segments(polygon.holes,
+                                                clockwise=not clockwise,
+                                                context=context)
